@@ -3,13 +3,15 @@
 
 # create environment private to this package
 # for caching (field names, binary checks)
-.dbffenv <- new.env(parent = emptyenv())
+# name of hidden environment: .ctrdataenv
+# see also ctrCache
+.ctrdataenv <- new.env()
 
 # check helper functions
 .onAttach <- function(libname, pkgname) {
 
   packageStartupMessage(
-    "Information on this package and how to use it:\n",
+    "\nInformation on this package and how to use it:\n",
     "https://cran.r-project.org/package=ctrdata\n\n",
     "Please respect the requirements and the copyrights of the\n",
     "clinical trial registers when using their information. Call\n",
@@ -19,12 +21,6 @@
     "https://www.isrctn.com/page/faqs#usingISRCTN\n"
   )
 
-  # check availabilities
-  packageStartupMessage("Checking helper binaries: ", appendLF = FALSE)
-  if (.Platform$OS.type == "windows") installCygwinWindowsTest()
-  checkBinary()
-  packageStartupMessage("done.")
-
   # return
   invisible()
 
